//
// Basic Vertex Lighting with a Texture
//

string description = "Basic Vertex Lighting with a Texture";

//////////////////////////////////////////
// UnTweakables, assigned automatically //
//////////////////////////////////////////

float4x4 WvpXf : WorldViewProjection < string UIWidget="None"; >;
float4x4 WorldXf   : World < string UIWidget="None"; >;
float4x4 WInvTrXf : WorldInverseTranspose < string UIWidget="None"; >;
float4x4 ViewInvXf : ViewInverse < string UIWidget="None"; >;

///////////////////////////////////////////////
//  Tweakables Appear in the Properties Pane //
///////////////////////////////////////////////

float4 LightDir : Direction <
    string UIName = "Light Direction";
	string Object = "DirectionalLight";
    string Space = "World";
> = {1.0f, -1.0f, 1.0f, 0.0f};

float4 LightColor : Diffuse <
	string UIWidget = "color";
    string UIName = "Diffuse Light";
    string Object = "DirectionalLight";
> = {1.0f, 1.0f, 1.0f, 1.0f};

float4 LightAmbient : Ambient <
	string UIWidget = "color";
    string UIName = "Ambient Light";
    string Space = "material";
> = {0.0f, 0.0f, 0.0f, 1.0f};

float4 MaterialDiffuse : Diffuse <
	string UIWidget = "color";
    string UIName = "Surface";
    string Space = "material";
> = {1.0f, 1.0f, 1.0f, 1.0f};

float4 MaterialSpecular : Specular <
	string UIWidget = "color";
	string UIName = "Surface Specular";
	string Space = "material";
> = {1.0f, 1.0f, 1.0f, 1.0f};

float Shininess : SpecularPower <
    string UIWidget = "slider";
    float UIMin = 1.0;
    float UIMax = 128.0;
    float UIStep = 1.0;
    string UIName = "Specular Power";
> = 30.0;

////////////////////////////////////
//  Texture  ///////////////////////
////////////////////////////////////

texture diffuseTexture : Diffuse <
	string ResourceName = "default_color.dds";
>;

sampler TextureSampler = sampler_state 
{
    texture = <diffuseTexture>;
    AddressU  = CLAMP;        
    AddressV  = CLAMP;
    // AddressW  = CLAMP;
    MIPFILTER = LINEAR;
    MINFILTER = LINEAR;
    MAGFILTER = LINEAR;
};

////////////////////////////////////
//  Structures Used ////////////////
////////////////////////////////////

struct vertexInput {
    float3 position				: POSITION;
    float3 normal				: NORMAL;
    float4 texCoordDiffuse		: TEXCOORD0;
};

struct vertexOutput {
    float4 hPosition		: POSITION;
    float4 texCoordDiffuse	: TEXCOORD0;
    float4 diffAmbColor		: COLOR0;
    float4 specCol			: COLOR1;
};

////////////////////////////////////
// Vertex Shader ///////////////////
////////////////////////////////////

vertexOutput VS_TransformAndTexture(vertexInput IN) 
{
    vertexOutput OUT = (vertexOutput)0;
    float4 Po = float4(IN.position.xyz,1.0); // object-coordinates position
    OUT.hPosition = mul(Po,WvpXf);
    OUT.texCoordDiffuse = IN.texCoordDiffuse;
	//calculate our vectors N, E, L, and H
	float3 Ew = ViewInvXf[3].xyz; // eye position in world coordinates
    float3 Pw = mul(Po, WorldXf).xyz; // world coordinates position
	float4 Nn = normalize(mul(IN.normal, WInvTrXf)); // normal vector
    float3 En = normalize(Ew - Pw); // eye vector
    float3 Ln = normalize( -LightDir.xyz); // light vector
    float3 Hn = normalize(En + Ln); // half angle vector
	// calculate the diffuse and specular contributions
    float diff = max(0 , dot(Nn,Ln));
    float spec = pow(max(0,dot(Nn,Hn)), Shininess);
    if( diff <= 0 ) {
        spec = 0;
    }
	//output diffuse
    float4 ambColor = MaterialDiffuse * LightAmbient;
    float4 diffColor = MaterialDiffuse * diff * LightColor ;
    OUT.diffAmbColor = diffColor + ambColor;
	//output specular
    float4 specColor = MaterialSpecular * LightColor * spec;
    OUT.specCol = specColor;
    return OUT;
}

////////////////////////////////////
// Pixel Shader ////////////////////
////////////////////////////////////

float4 PS_Textured( vertexOutput IN): COLOR
{
	float4 diffuseTexture = tex2D( TextureSampler, IN.texCoordDiffuse );
	return IN.diffAmbColor * diffuseTexture + IN.specCol;
}

////////////////////////////////////
// Complete Technique //////////////
////////////////////////////////////

technique textured
{
    pass p0 
    {		
		VertexShader = compile vs_1_1 VS_TransformAndTexture();
		PixelShader  = compile ps_1_1 PS_Textured();
    }
}

//
// $Id: //sw/devtools/SDK/9.5/SDK/MEDIA/HLSL/new_material.fx#1 $
//

////////////////////////////////////
//////////////////////////// eof ///
////////////////////////////////////
